from dataclasses import dataclass
from logging import getLogger
from typing import Literal
from aiohttp import ClientSession
from ..types.api_ret.price_history import PriceHistoryAPIRet
from ..lib.utils import get_nindex_from_url, get_nindex_oid, get_pid_from_url
from ..types.setting import BigGoMCPSetting

logger = getLogger(__name__)

DAYS = Literal["90", "80", "365", "730"]


@dataclass(slots=True)
class PriceHistoryRet:
    description: PriceHistoryAPIRet
    graph_link: str


class PriceHistoryService:

    def __init__(self, setting: BigGoMCPSetting):
        self._setting = setting

    def _get_history_id(self, nindex: str, pid: str) -> str:
        return f"{nindex}-{pid}"

    def graph_link(self, history_id: str) -> str:
        item_info = get_nindex_oid(history_id)
        return f"https://imbot-dev.biggo.dev/chart?nindex={item_info.nindex}&oid={item_info.oid}&lang={self._setting.graph_language}"

    async def _get_price_history(self, history_id: str,
                                 days: int) -> PriceHistoryAPIRet | None:
        url = "https://extension.biggo.com/api/product_price_history.php"
        body = {"item": [history_id], "days": days}

        logger.debug("call price history, body: %s", body)

        async with ClientSession() as session:
            async with session.post(url=url, json=body) as resp:
                if resp.status >= 400:
                    err_msg = f"price history api error: {await resp.text()}"
                    logger.error(err_msg)
                    raise ValueError(err_msg)

                if (data := await resp.json()).get("result") == False:
                    return None
                else:
                    return PriceHistoryAPIRet.model_validate(data)

    async def history_with_history_id(
        self,
        history_id: str,
        days: DAYS,
    ) -> PriceHistoryRet | None:
        description = await self._get_price_history(history_id, int(days))
        if description is None:
            return None
        else:
            url = self.graph_link(history_id)
            return PriceHistoryRet(description=description, graph_link=url)

    async def history_with_url(self, url: str,
                               days: DAYS) -> PriceHistoryRet | None:

        if (nindex := await get_nindex_from_url(url)) is None:
            logger.warning("nindex not found, url: %s", url)
            return

        if (pid := await get_pid_from_url(nindex=nindex, url=url)) is None:
            logger.warning("product id not found, nindex: %s, url: %s", nindex,
                           url)
            return

        history_id = self._get_history_id(nindex=nindex, pid=pid)
        resp = await self._get_price_history(history_id=history_id,
                                             days=int(days))

        if resp is None and nindex in ["tw_mall_shopeemall", "tw_bid_shopee"]:
            nindex = "tw_mall_shopeemall" if nindex == "tw_bid_shopee" else "tw_mall_shopeemall"
            history_id = self._get_history_id(nindex=nindex, pid=pid)
            resp = await self._get_price_history(
                history_id=history_id,
                days=int(days),
            )

        if resp is not None:
            graph_link = self.graph_link(history_id)
            return PriceHistoryRet(description=resp, graph_link=graph_link)
        else:
            return None



from typing import Self
from pydantic import BaseModel, model_validator
from .api_ret.price_history import PriceHistoryAPIRet
from .api_ret.product_search import ProductSearchAPIRet


class BaseToolResponse(BaseModel):

    def slim_dump(self) -> str:
        return self.model_dump_json(exclude_none=True)


class ProductSearchToolResponse(BaseToolResponse):
    product_search_result: ProductSearchAPIRet
    reason: str | None = None

    @model_validator(mode='after')
    def post_init(self) -> Self:
        if len(self.product_search_result.list) == 0:
            self.reason = """
            No results found. Possible reasons:
            1. This search is much more complex than a simple product search.
            2. The user is asking things related to product specifications.

            If the problems might be related to the points listed above,
            please use the 'spec_search' tool and try again.
            """

        return self


class PriceHisotryGraphToolResponse(BaseToolResponse):
    price_history_graph: str


class PriceHistoryToolResponse(BaseToolResponse):
    price_history_description: PriceHistoryAPIRet
    price_history_graph: str


class SpecIndexesToolResponse(BaseToolResponse):
    indexes: list[str]


class SpecMappingToolResponse(BaseToolResponse):
    mappings: dict
    example_document: dict
    note: str = """
Specifications are under the 'specs' field
Example fields paths:
- specs.physical_specs.weight
- specs.technical_specs.water_resistance.depth
- specs.sensors.gyroscope
"""


class SpecSearchToolResponse(BaseToolResponse):
    hits: list[dict]
    reason: str | None = None

    @model_validator(mode='after')
    def post_init(self) -> Self:
        if len(self.hits) == 0:
            self.reason = """
            No results found. Possible reasons:
            1. You have no clue about the mapping.
            2. You have not used the 'spec_mapping' tool to get the mapping of the index.

            Do you really know what you are doing?
            You MUST think about this before you tell the user that no results are found.
            You MUST be ABSOLUTELY sure that you understand the mapping of the index, and that the
            search criteria is correct, and that there is TRULY no result.

            If you have any doubt, please use the 'spec_mapping' tool to get the mapping of the index,
            and try again.
            The 'spec_mapping' tool will give you the mapping of the index, and an example document.
            'spec_mapping' is the only way to understand the mapping of the index.
            It is the best tool to use before you search the index.
            """

        return self



import pytest
from biggo_mcp_server.services.spec_search import SpecMappingRet, SpecSearchService
from biggo_mcp_server.types.setting import BigGoMCPSetting
from .helper import *


@pytest.mark.asyncio
async def test_spec_search_service_indexes(setting: BigGoMCPSetting):
    service = SpecSearchService(setting)
    async with service.session():
        indexes = await service.spec_indexes()
        assert isinstance(indexes, list)


@pytest.mark.asyncio
async def test_spec_search_service_mapping(setting: BigGoMCPSetting):
    service = SpecSearchService(setting)
    async with service.session():
        indexes = await service.spec_indexes()
        mapping = await service.spec_mapping(indexes[0])
        assert isinstance(mapping, SpecMappingRet)


@pytest.mark.asyncio
async def test_spec_search_service_search(setting: BigGoMCPSetting):
    service = SpecSearchService(setting)
    async with service.session():
        indexes = await service.spec_indexes()
        hits = await service.search(indexes[0], {"query": {"match_all": {}}})
        assert isinstance(hits, list)

        with pytest.raises(ValueError):
            await service.search(indexes[0], {
                "query": {
                    "match_all": {}
                },
                "size": 11
            })



import json
from logging import getLogger
from typing import Annotated
from mcp.server.fastmcp import Context
from pydantic import Field
from ..types.responses import SpecIndexesToolResponse, SpecMappingToolResponse, SpecSearchToolResponse
from ..lib.utils import get_setting
from ..services.spec_search import SpecSearchService

logger = getLogger(__name__)


async def spec_indexes(
    ctx: Context,
) -> Annotated[str, Field(description="List of Elasticsearch indexes")]:
    """Elasticsearch Indexes for Product Specification.

    It is REQUIRED to use this tool first before running any specification search.
    """
    logger.info("spec indexes")
    setting = get_setting(ctx)
    service = SpecSearchService(setting)
    async with service.session():
        indexes = await service.spec_indexes()
    return SpecIndexesToolResponse(indexes=indexes).slim_dump()


async def spec_mapping(
    ctx: Context,
    index: Annotated[str,
                     Field(description="""
                          Elasticsearch index

                          Steps to obtain this argument.
                          1. Use 'spec_indexes' tool to get the list of indexes
                          2. Choose the most relevant index
                          """)],
) -> Annotated[
        str,
        Field(description="Elasticsearch mappings plus an example document")]:
    """Elasticsearch Mapping For Product Specification.

    Use this tool after you have the index, and need to know the mapping in order to query the index.
    Available indexes can be obtained by using the 'spec_indexes' tool.
    """
    logger.info("spec mapping, index: %s", index)
    setting = get_setting(ctx)
    service = SpecSearchService(setting)
    try:
        async with service.session():
            mapping = await service.spec_mapping(index)
    except Exception as e:
        raise Exception(
            f"You must know the available indexes before using this tool. Error: {e}"
        )
    return SpecMappingToolResponse(
        mappings=mapping.mappings,
        example_document=mapping.example_document).slim_dump()


async def spec_search(
    ctx: Context,
    index: Annotated[str,
                     Field(description="""
                          Elasticsearch index

                          Steps to obtain this argument.
                          1. Use 'spec_indexes' tool to get the list of indexes
                          2. Choose the most relevant index
                          """)],
    elasticsearch_query: Annotated[
        str | dict,
        Field(description="""
              Elasticsearch query

              Please use the 'spec_mapping' tool to get the mapping of the index, before using this tool.

              Size must be less than or equal to 10.
              """,
              examples=[{
                  "query": {
                      "range": {
                          "specs.physical_specifications.dimensions.height": {
                              "gte": 1321,
                              "lte": 2321
                          }
                      }
                  },
                  "size":
                      5,
                  "sort": [{
                      "specs.physical_specifications.dimensions.height": "asc"
                  }]
              }])],
) -> str:
    """Product Specification Search. 

    Index mapping must be aquired before using this tool.
    Use 'spec_mapping' tool to get the mapping of the index.
    """
    logger.info("spec search, index: %s, query: %s", index, elasticsearch_query)

    setting = get_setting(ctx)
    service = SpecSearchService(setting)
    try:
        async with service.session():
            if isinstance(elasticsearch_query, str):
                query: dict = json.loads(elasticsearch_query)
            else:
                query = elasticsearch_query
            hits = await service.search(index, query)
    except Exception as e:
        raise Exception(
            f"You must know the mapping of the index before using this tool. Error: {e}"
        )
    return SpecSearchToolResponse(hits=hits).slim_dump()



from logging import getLogger
from typing import Annotated
from mcp.server.fastmcp import Context
from pydantic import Field

from ..services.product_search import ProductSearchService
from ..types.responses import ProductSearchToolResponse
from ..lib.utils import get_setting

logger = getLogger(__name__)


async def product_search(
    ctx: Context,
    query: Annotated[
        str,
        Field(
            description="""Search query""",
            examples=["iphone", "護唇膏"],
        ),
    ],
) -> str:
    """Product Search"""
    logger.info("product search, query: %s", query)

    setting = get_setting(ctx)
    service = ProductSearchService(setting)
    ret = await service.search(query)

    return ProductSearchToolResponse(product_search_result=ret).slim_dump()



from typing import Annotated
from mcp.server.fastmcp import Context
from pydantic import Field
from biggo_mcp_server.types.setting import Regions
from ..lib.utils import get_setting


def get_current_region(
    ctx: Context,
) -> Annotated[Regions, Field(description="Current region")]:
    """
    Get the current region setting.
    """
    setting = get_setting(ctx)
    return setting.region



import pytest
from os import environ
from biggo_mcp_server.types.setting import BigGoMCPSetting

ES_PROXY_URL = environ.get(
    "ES_PROXY_URL", "https://api.d.cloud.biggo.com/api/v1/mcp-es-proxy/")
AUTH_TOKEN_URL = environ.get(
    "AUTH_TOKEN_URL", "https://api-auth.dev.cloud.biggo.com/auth/v1/token")

CLIENT_ID = environ.get("CLIENT_ID", None)
CLIENT_SECRET = environ.get("CLIENT_SECRET", None)


@pytest.fixture
def setting():
    setting = BigGoMCPSetting(client_id=CLIENT_ID,
                              client_secret=CLIENT_SECRET,
                              es_proxy_url=ES_PROXY_URL,
                              auth_token_url=AUTH_TOKEN_URL,
                              es_verify_certs=False,
                              auth_verify_certs=False)
    return setting



from pydantic import BaseModel, Field
"""
Example:

{
"biggo": [
    {
    "id": "tw_pec_momoshop",
    "region": "tw"
    }
],
"honey": []
}
"""


class NindexFromUrlItem(BaseModel):
    id: str
    region: str


class NindexFromUrlAPIRet(BaseModel):
    biggo: list[NindexFromUrlItem] = Field(default_factory=list)

    @property
    def nindex(self) -> str | None:
        if len(self.biggo) == 0:
            return

        return self.biggo[0].id



from __future__ import annotations

from typing import Any, List

from pydantic import BaseModel, Field


class Multiple(BaseModel):
    min_price: float | None = None
    max_price: float | None = None
    model_id: str | None = None
    title: str | None = None
    current_price: float | None = None


class Store(BaseModel):
    # image: str
    # link: str
    name: str | None = None
    discount_info: str | None = None
    # rate_desc: str | None = None
    # is_cashback: bool


class Shop(BaseModel):
    name: str | None = None
    username: str | None = None
    uid: str | None = None
    location: str | None = None
    # seller_credit: int | None = None


class ListItem(BaseModel):
    nindex: str | None = None
    oid: str | None = None
    # subscribe_id: Any
    history_id: str | None = None
    item_id: str | None = None
    # is_ad: bool
    # is_group: bool
    # is_offline: bool
    # is_notfound: bool
    # is_expired: bool
    # is_adult: bool
    # is_multiple_product: bool
    # is_subscribe: bool
    title: str | None = None
    purl: str | None = None
    # affurl: str
    image: str | None = None
    # gallery_count: int
    # origin_image: str
    cata: List | None = None
    symbol: str | None = None
    currency: str | None = None
    multiple: Multiple | None = None
    price: float | None = None
    price_range_min: Any | None = None
    price_range_max: Any | None = None
    count_result_store: Any | None = None
    count_result_product: Any | None = None
    store: Store | None = None
    has_shop: bool | None = None
    shop: Shop | None = None
    price_diff_real: float | None = None
    product_nindex_price: List | None = None
    # subscribe_tags: List
    # subscribe_time: Any


class BiggoCItem(BaseModel):
    key: str | None = None
    value: str | None = None
    time: int | None = None


class ProductSearchAPIRet(BaseModel):
    # result: bool
    # total: int
    # total_page: int
    # pure_total: int
    # ec_count: int
    # mall_count: int
    # bid_count: int
    # size: int
    # took: int
    # is_shop: bool
    # is_suggest_query: bool
    # is_ypa: bool
    # is_adsense: bool
    # q_suggest: str
    # arr_suggest: List[str]
    # offline_count: int
    # spam_count: int
    # promo: List
    # filter: Dict[str, Any]
    # top_ad_count: int
    # group: Any
    # recommend_group: List
    list: List[ListItem] = Field(default_factory=list)
    # biggo_c: List[BiggoCItem]
    low_price: float | None = None
    high_price: float | None = None



from typing import Generic, TypeVar
from pydantic import BaseModel

T = TypeVar("T")


class ErrorContext(BaseModel):
    code: int
    message: str


class BigGoAPIRet(BaseModel, Generic[T]):
    result: bool
    data: T | None = None
    error: ErrorContext | None = None



from logging import getLogger
from mcp.server.fastmcp import FastMCP
from ..types.setting import BigGoMCPSetting

logger = getLogger(__name__)


class BigGoMCPServer(FastMCP):

    def __init__(self, setting: BigGoMCPSetting):
        super().__init__("BigGo MCP Server")
        self._biggo_setting = setting

    @property
    def biggo_setting(self) -> BigGoMCPSetting:
        return self._biggo_setting



from __future__ import annotations

from typing import List

from pydantic import BaseModel, RootModel


class PriceHistoryItem(BaseModel):
    x: int
    y: int


class Days(BaseModel):
    days: int
    max_price: float
    min_price: float
    up_times: int
    down_times: int
    last_continue_times: int
    last_increase_status: bool


class Statistics(BaseModel):
    days730: Days | None = None
    days365: Days | None = None
    days180: Days | None = None
    days90: Days | None = None
    days30: Days | None = None
    days7: Days | None = None


class PriceHistoryAttributes(BaseModel):
    symbol: str
    currency: str
    nindex: str
    oid: str
    current_price: float
    # datetime_format: str
    price_history: List[PriceHistoryItem]
    # purl: str | None = None
    url: str | None = None
    title: str
    nindex_name: str
    icon: str
    statistics: Statistics
    state: str


class PriceHistoryAPIRet(RootModel):
    root: dict[str, PriceHistoryAttributes]



from pydantic import BaseModel, Field, RootModel


class SpecIndexesAPIItem(BaseModel):
    index: str


class SpecIndexesAPIRet(RootModel):
    root: list[SpecIndexesAPIItem] = Field(default_factory=list)


class SpecMappingAPIRet(RootModel):
    """
    Example:
    {
        "spec_grouping_bidet_toilet_seat": {
            "mappings": {
                "properties": {}
            }
        }
    }
    """
    root: dict[str, dict] = Field(default_factory=dict)


class SpecSearchAPIRet(RootModel):
    root: list[dict]



from enum import StrEnum
from pydantic import BaseModel


class GraphLanguage(StrEnum):
    TW = "tw"
    EN = "en"


class LogLevel(StrEnum):
    DEBUG = "DEBUG"
    INFO = "INFO"
    WARNING = "WARNING"
    ERROR = "ERROR"
    CRITICAL = "CRITICAL"


class Regions(StrEnum):
    ID = "ID"
    VN = "VN"
    TH = "TH"
    PH = "PH"
    MY = "MY"
    IN = "IN"
    US = "US"
    TW = "TW"
    HK = "HK"
    JP = "JP"
    SG = "SG"


class Domains(StrEnum):
    ID = "biggo.id"
    VN = "vn.biggo.com"
    TH = "biggo.co.th"
    PH = "ph.biggo.com"
    MY = "my.biggo.com"
    IN = "biggo.co.in"
    US = "biggo.com"
    TW = "biggo.com.tw"
    HK = "biggo.hk"
    JP = "biggo.jp"
    SG = "biggo.sg"


REGION_DOMAIN_MAP: dict[Regions, Domains] = {
    Regions.ID: Domains.ID,
    Regions.VN: Domains.VN,
    Regions.TH: Domains.TH,
    Regions.PH: Domains.PH,
    Regions.MY: Domains.MY,
    Regions.IN: Domains.IN,
    Regions.US: Domains.US,
    Regions.TW: Domains.TW,
    Regions.HK: Domains.HK,
    Regions.JP: Domains.JP,
    Regions.SG: Domains.SG,
}


class BigGoMCPSetting(BaseModel):
    """
    BigGo MCP Server settings
    """

    client_id: str | None = None
    client_secret: str | None = None

    region: Regions = Regions.TW

    log_level: LogLevel = LogLevel.INFO

    es_proxy_url: str = "https://mcp-es-proxy.d.cloud.biggo.com"
    es_verify_certs: bool = True

    auth_token_url: str = "https://api.biggo.com/auth/v1/token"
    auth_verify_certs: bool = True

    @property
    def domain(self) -> Domains:
        return REGION_DOMAIN_MAP[self.region]

    @property
    def graph_language(self) -> GraphLanguage:
        if self.region == Regions.TW:
            return GraphLanguage.TW
        else:
            return GraphLanguage.EN



from logging import getLogger
import base64
from aiohttp import ClientSession
from ..types.api_ret.auth_token import AuthTokenRet
from async_lru import alru_cache

logger = getLogger(__name__)


@alru_cache(maxsize=1, ttl=60)
async def get_access_token(
    client_id: str,
    client_secret: str,
    ssl: bool = True,
    endpoint: str = "https://api.biggo.com/auth/v1/token",
) -> str:
    """Get access token with client credentials"""

    params = {
        "grant_type": "client_credentials",
    }
    credentials = f"{client_id}:{client_secret}".encode()
    authorization = base64.b64encode(credentials).decode()
    headers = {
        "Authorization": f"Basic {authorization}",
        "Content-Type": "application/x-www-form-urlencoded"
    }

    async with ClientSession() as session:
        async with session.post(
                url=endpoint,
                headers=headers,
                params=params,
                ssl=ssl,
        ) as resp:
            if resp.status >= 400:
                err_msg = f"get access token error, {await resp.text()}"
                logger.error(err_msg)
                raise ValueError(err_msg)

            data = await resp.json()
            return AuthTokenRet.model_validate(data).access_token



from dataclasses import dataclass
from biggo_mcp_server.types.api_ret.product_search import ProductSearchAPIRet


@dataclass(slots=True)
class Statistics:
    original: int
    cleaned: int


def print_length_comparison(original: int, cleaned: int):
    print(f"""Original length: {original}, Cleaned length: {cleaned}. 
        Current Percentage = ({(cleaned/original) * 100})%
        Original / Cleaned = {original/cleaned}
        """)


def product_search_api_ret() -> Statistics:
    print("== product_search_api_ret ==")
    # original response
    with open("./.data/original_prod_search_ret.json", "r") as f:
        original = f.read()

    # cleaned
    clean_data = ProductSearchAPIRet.model_validate_json(original)
    clean_data = clean_data.model_dump_json(exclude_none=True)

    # compaire
    print_length_comparison(original=len(original), cleaned=len(clean_data))

    return Statistics(original=len(original), cleaned=len(clean_data))


def main():
    product_search_statistic = product_search_api_ret()

    print("== Total length comparison ==")
    original = product_search_statistic.original
    cleaned = product_search_statistic.cleaned
    print_length_comparison(original=original, cleaned=cleaned)


if __name__ == "__main__":
    main()



from dataclasses import dataclass
from logging import getLogger
import re
from typing import Any
from urllib.parse import quote_plus, urlparse, parse_qs
from mcp.server.fastmcp import Context
from ..types.setting import BigGoMCPSetting
from ..types.api_ret.ec_list import EcListAPIData, EcListAPIRet, EcListPattern
from ..types.api_ret.ninde_from_url import NindexFromUrlAPIRet
from aiohttp import ClientSession

logger = getLogger(__name__)


def get_setting(ctx: Context) -> BigGoMCPSetting:
    server: BigGoMCPServer = ctx.fastmcp  # type: ignore
    return server.biggo_setting


@dataclass(slots=True)
class NindexOID:
    nindex: str
    oid: str


def get_nindex_oid(inpt: str) -> NindexOID:
    """
    Arguments:
    - inpt: structure <nindex>-<oid>
    """
    temp = inpt.split("-", maxsplit=1)
    return NindexOID(nindex=temp[0], oid=temp[1])


def get_query_variable(url: str, field: str) -> str | None:
    try:
        url_obj = urlparse(url)
        query_params = parse_qs(url_obj.query)
        return query_params.get(field, [None])[0]
    except Exception:
        return None


def get_url_match(url: str, regex: str) -> re.Match | None:
    return re.search(regex, url)


@dataclass(slots=True)
class GetIdWithRegexpRet:
    match: re.Match | None
    pid: str | None


def get_id_with_regexp(
        url: str,
        regex: str,
        template: str = "%1",
        config: EcListPattern | None = None) -> GetIdWithRegexpRet:
    match = get_url_match(url, regex)
    pid = None

    if match:
        pid = match.group(1)

    # Replace template
    if template and match:
        pid = template.replace("%1", match.group(1))
        if "%2" in template and len(match.groups()) > 1:
            pid = template.replace("%2", match.group(2))
            if "%1" in pid:
                pid = pid.replace("%1", match.group(1))

    # Padding
    if (config and config.len and isinstance(pid, str) and "%p" in pid and
            config.len > len(pid)):
        padding_length = config.len - (len(pid) - 2)
        padding = config.pad * padding_length
        pid = pid.replace("%p", padding)

    return GetIdWithRegexpRet(match=match, pid=pid)


def parse_url(url: str, nindex: str, config: EcListPattern) -> str | None:
    regex = config.match
    query = config.query
    template = config.template

    # init parameter
    match = []
    pid: str | None = ""

    # check regexp
    if regex:
        if not isinstance(regex, list):
            regex = [regex]
        if not isinstance(template, list):
            template = [template]

        tmp: list[GetIdWithRegexpRet] = []
        for idx, re_pattern in enumerate(regex):
            template_to_use = template[idx] if idx < len(
                template) else template[0]
            result = get_id_with_regexp(url, re_pattern, template_to_use,
                                        config)
            if result.pid:
                tmp.append(result)

        if len(tmp) > 0:
            if tmp[0].match:
                match = [
                    tmp[0].match.group(i)
                    for i in range(len(tmp[0].match.groups()) + 1)
                ]
            pid = tmp[0].pid

    # check url search query
    if query:
        query_pid = get_query_variable(url, query)
        pid = query_pid if query_pid is not None else pid

    # custom rule
    if nindex == "tw_pmall_rakuten":
        if len(match) > 2 and match[1] and match[2]:
            return f"{match[1]}_{match[2].upper()}"

    if config.uppercase and pid:
        pid = pid.upper()

    if config.lowercase and pid:
        pid = pid.lower()

    return pid


def get_product_id(ec_list: dict[str, EcListPattern], nindex: str,
                   url: str) -> str | None:
    pid: Any = ""

    if nindex in ec_list:
        current = ec_list[nindex]
        pid = parse_url(url, nindex, current)

    if isinstance(pid, dict) and 'pid' in pid:
        return pid['pid']
    elif isinstance(pid, str):
        return pid
    else:
        return ""


async def get_nindex_from_url(url: str) -> str | None:
    url = f"https://extension.biggo.com/api/store.php?method=domain_lookup&domain={quote_plus(url)}"
    logger.debug("get_nindex_from_url, url: %s", url)
    async with ClientSession() as session:
        async with session.get(url) as resp:
            return NindexFromUrlAPIRet.model_validate(await resp.json()).nindex


async def get_ec_list() -> EcListAPIData | None:
    url = "https://extension.biggo.com/api/eclist.php"
    async with ClientSession() as session:
        async with session.get(url) as resp:
            data = EcListAPIRet.model_validate(await resp.json())
            return data.data


async def get_pid_from_url(nindex: str, url: str) -> str | None:
    ec_list = await get_ec_list()
    if not ec_list:
        logger.warning("ec list is empty")
        return

    region = nindex.split("_")[0]
    if region not in ec_list:
        logger.warning("region not in ec list. region: %s, nindex: %s", region,
                       nindex)
        return

    return get_product_id(ec_list[region], nindex, url)



from urllib.parse import quote
from aiohttp import ClientSession
from ..types.api_ret.product_search import ProductSearchAPIRet
from ..types.setting import BigGoMCPSetting
from logging import getLogger

logger = getLogger(__name__)


class ProductSearchService:

    def __init__(self, setting: BigGoMCPSetting):
        self._setting = setting

    async def search(self, query: str) -> ProductSearchAPIRet:
        url = f"https://api.biggo.com/api/v1/spa/search/{query}/product"

        headers = {
            "Content-Type": "application/json",
            "site": self._setting.domain,
            "region": self._setting.region.lower(),
        }
        logger.debug("product search, url: %s, headers: %s", url, headers)

        async with ClientSession() as session:
            async with session.get(url, headers=headers) as resp:
                if resp.status >= 400:
                    err_msg = f"product search api error: {await resp.text()}"
                    logger.error(err_msg)
                    raise ValueError(err_msg)

                return ProductSearchAPIRet.model_validate(await resp.json())



from __future__ import annotations

from typing import Any, Literal

from pydantic import BaseModel, ConfigDict

from .common import BigGoAPIRet


class EcListPattern(BaseModel):
    ptn: str
    bg_mark: str
    match: str | list[str] = ""
    template: str | list[str] = "%1"
    query: str = ""
    uppercase: bool = False
    lowercase: bool = False
    len: int | None = None
    pad: Any = ""


EcListRegion = Literal["tw", "thai", "jp", "hk", "id", "my", "ph", "sg", "us",
                       "vn"] | str
EcListAPIData = dict[EcListRegion, dict[str, EcListPattern]]


class EcListAPIRet(BigGoAPIRet[EcListAPIData]):
    pass



from contextlib import asynccontextmanager
from dataclasses import dataclass
from logging import getLogger
from elasticsearch8 import AsyncElasticsearch
from ..types.api_ret.spec import SpecIndexesAPIRet, SpecMappingAPIRet, SpecSearchAPIRet
from ..lib.access_token import get_access_token
from ..types.setting import BigGoMCPSetting

logger = getLogger(__name__)


@dataclass(slots=True)
class SpecMappingRet:
    mappings: dict
    example_document: dict


class SpecSearchService:

    def __init__(self, setting: BigGoMCPSetting):
        self._setting = setting

    @asynccontextmanager
    async def session(self):
        access_token = await self._get_access_token()
        self._es_conn = AsyncElasticsearch(
            hosts=self._setting.es_proxy_url,
            verify_certs=self._setting.es_verify_certs,
            bearer_auth=access_token,
        )
        try:
            yield
        finally:
            await self._es_conn.close()

    async def _get_access_token(self) -> str:
        if self._setting.client_id is None or self._setting.client_secret is None:
            err_msg = "Client ID or Client Secret is not set"
            logger.error(err_msg)
            raise ValueError(err_msg)

        return await get_access_token(
            client_id=self._setting.client_id,
            client_secret=self._setting.client_secret,
            endpoint=self._setting.auth_token_url,
            ssl=self._setting.auth_verify_certs,
        )

    async def spec_indexes(self) -> list[str]:
        resp = await self._es_conn.cat.indices(index="spec*",
                                               format="json",
                                               h="index")
        data = SpecIndexesAPIRet.model_validate(resp.body)
        return [item.index for item in data.root]

    async def spec_mapping(self, index: str) -> SpecMappingRet:
        resp = await self._es_conn.indices.get_mapping(index=index)
        data = SpecMappingAPIRet.model_validate(resp.body)
        mappings = data.root[index]['mappings']

        resp = await self._es_conn.search(index=index, size=1)
        example_document = resp.body['hits']['hits'][0]['_source']

        return SpecMappingRet(mappings=mappings,
                              example_document=example_document)

    async def search(self, index: str, query: dict) -> list[dict]:
        size = query.get('size', None)
        if size is not None and size > 10:
            raise ValueError("Size must be less than or equal to 10")
        else:
            query['size'] = 5

        logger.debug("Actual search args, index: %s, query: %s", index, query)

        resp = await self._es_conn.search(index=index, body=query)
        data = SpecSearchAPIRet.model_validate(resp.body['hits']['hits'])
        return data.root



import argparse
import asyncio
from logging import getLogger
from .types.setting import BigGoMCPSetting, LogLevel, Regions
from .lib.server_setup import create_server
from dataclasses import dataclass

logger = getLogger(__name__)


@dataclass()
class Args:
    region: Regions
    client_id: str | None
    client_secret: str | None
    log_level: LogLevel
    es_proxy_url: str
    auth_token_url: str
    es_verify_certs: bool
    auth_verify_certs: bool


async def start():
    args = argparse.ArgumentParser()
    args.add_argument("--region",
                      type=Regions,
                      choices=Regions,
                      default=Regions.TW)
    args.add_argument("--client-id", type=str, default=None)
    args.add_argument("--client-secret", type=str, default=None)
    args.add_argument("--log-level",
                      type=LogLevel,
                      choices=LogLevel,
                      default=LogLevel.INFO)
    args.add_argument("--es-proxy-url",
                      type=str,
                      default="https://api.biggo.com/api/v1/mcp-es-proxy/")
    args.add_argument("--es-verify-certs", type=bool, default=True)
    args.add_argument("--auth-token-url",
                      type=str,
                      default="https://api.biggo.com/auth/v1/token")
    args.add_argument("--auth-verify-certs", type=bool, default=True)

    args = args.parse_args(namespace=Args)

    setting = BigGoMCPSetting(
        region=args.region,
        client_id=args.client_id,
        client_secret=args.client_secret,
        log_level=args.log_level,
        es_proxy_url=args.es_proxy_url,
        auth_token_url=args.auth_token_url,
        es_verify_certs=args.es_verify_certs,
        auth_verify_certs=args.auth_verify_certs,
    )

    server = await create_server(setting)

    logger.info("Starting BigGo MCP Server")

    await server.run_stdio_async()


def main():
    asyncio.run(start())


if __name__ == "__main__":
    main()



from logging import getLogger

from ..tools.util import get_current_region
from ..tools.spec_search import (
    spec_indexes,
    spec_mapping,
    spec_search,
)
from ..tools.price_history import (
    price_history_graph,
    price_history_with_history_id,
    price_history_with_url,
)
from ..tools.product_search import product_search
from .log import setup_logging
from ..types.setting import BigGoMCPSetting
from .server import BigGoMCPServer

logger = getLogger(__name__)


async def create_server(setting: BigGoMCPSetting) -> BigGoMCPServer:
    server = BigGoMCPServer(setting)
    setup_logging(setting.log_level)

    # Product Search
    server.add_tool(product_search)

    # Price History
    server.add_tool(price_history_graph)
    server.add_tool(price_history_with_history_id)
    server.add_tool(price_history_with_url)

    # Spec Search
    server.add_tool(spec_indexes)
    server.add_tool(spec_mapping)
    server.add_tool(spec_search)

    # Util
    server.add_tool(get_current_region)

    return server



from logging import getLogger
from ..types.setting import LogLevel

logger = getLogger(__name__)


def setup_logging(log_level: LogLevel = LogLevel.INFO):
    logger = getLogger("biggo_mcp_server")
    logger.setLevel(log_level)
    msg = "BigGo MCP Server logging setup, log_level: %s"
    match log_level:
        case LogLevel.DEBUG:
            logger.debug(msg, log_level)
        case LogLevel.INFO:
            logger.info(msg, log_level)
        case LogLevel.WARNING:
            logger.warning(msg, log_level)
        case LogLevel.ERROR:
            logger.error(msg, log_level)
        case LogLevel.CRITICAL:
            logger.critical(msg, log_level)



from logging import getLogger
from typing import Annotated
from mcp.server.fastmcp import Context
from pydantic import Field
from ..types.responses import PriceHisotryGraphToolResponse, PriceHistoryToolResponse
from ..services.price_history import DAYS, PriceHistoryService
from ..lib.utils import get_setting

logger = getLogger(__name__)


def price_history_graph(
    ctx: Context,
    history_id: Annotated[
        str,
        Field(description="""
              Product History ID
              Here are a few steps to obtain this argument.
              1. Use 'product_search' tool to retrive a list of products
              2. Find the most relevant product.
              3. The product should have a field called 'history_id', use it as the value for this argument
              """,
              examples=[
                  "tw_pmall_rakuten-nwdsl_6MONJRBOO", "tw_pec_senao-1363332"
              ])],
) -> str:
    """Link That Visualizes Product Price History"""

    logger.info("price history graph, history_id: %s", history_id)

    setting = get_setting(ctx)
    service = PriceHistoryService(setting)
    url = service.graph_link(history_id)

    return PriceHisotryGraphToolResponse(
        price_history_graph=f"![Price History Graph]({url})").slim_dump()


async def price_history_with_history_id(
    ctx: Context,
    history_id: Annotated[
        str,
        Field(description="""
              Product History ID
              Here are a few steps to obtain this argument.
              1. Use 'product_search' tool to retrive a list of products
              2. Find the most relevant product.
              3. The product should have a field called 'history_id', use it as the value for this argument
              """,
              examples=[
                  "tw_pmall_rakuten-nwdsl_6MONJRBOO", "tw_pec_senao-1363332"
              ])],
    days: Annotated[DAYS, Field(description="History range")],
) -> str:
    """Product Price History With History ID"""

    logger.info("price history with history id, history_id: %s, days: %s",
                history_id, days)

    setting = get_setting(ctx)
    service = PriceHistoryService(setting)
    ret = await service.history_with_history_id(history_id=history_id,
                                                days=days)
    if ret is None:
        return "No price history found"

    return PriceHistoryToolResponse(
        price_history_description=ret.description,
        price_history_graph=f"![Price History Graph]({ret.graph_link})"
    ).slim_dump()


async def price_history_with_url(
    ctx: Context,
    days: Annotated[DAYS, Field(description="History range")],
    url: Annotated[str, Field(description="Product URL")],
) -> str:
    """Product Price History With URL"""

    logger.info("price history with url, url: %s, days: %s", url, days)

    setting = get_setting(ctx)
    service = PriceHistoryService(setting)
    ret = await service.history_with_url(url=url, days=days)
    if ret is None:
        return "No price history found"

    return PriceHistoryToolResponse(
        price_history_description=ret.description,
        price_history_graph=f"![Price History Graph]({ret.graph_link})"
    ).slim_dump()



"""
Just check if the tools can run without errors.
"""

from biggo_mcp_server.tools.price_history import (
    price_history_graph,
    price_history_with_history_id,
    price_history_with_url,
)
from biggo_mcp_server.tools.product_search import product_search
import pytest
from unittest.mock import MagicMock

from biggo_mcp_server.tools.util import get_current_region
from biggo_mcp_server.types.setting import BigGoMCPSetting, Regions
from .helper import *


def test_get_current_region(setting: BigGoMCPSetting):

    ctx = MagicMock()
    ctx.fastmcp = MagicMock()
    ctx.fastmcp.biggo_setting = setting

    region = get_current_region(ctx)

    assert isinstance(region, Regions)


@pytest.mark.asyncio
async def test_get_history(setting: BigGoMCPSetting):

    ctx = MagicMock()
    ctx.fastmcp = MagicMock()
    ctx.fastmcp.biggo_setting = setting

    history = await price_history_with_history_id(
        ctx=ctx,
        history_id="tw_pmall_rakuten-nwdsl_6MONJRBOO",
        days="90",
    )

    assert isinstance(history, str)


@pytest.mark.asyncio
async def test_get_history_with_url(setting: BigGoMCPSetting):

    ctx = MagicMock()
    ctx.fastmcp = MagicMock()
    ctx.fastmcp.biggo_setting = setting

    history = await price_history_with_url(
        ctx=ctx,
        url="https://www.momoshop.com.tw/goods/GoodsDetail.jsp?i_code=13660781",
        days="90",
    )
    assert isinstance(history, str)


@pytest.mark.asyncio
async def test_product_search(setting: BigGoMCPSetting):

    ctx = MagicMock()
    ctx.fastmcp = MagicMock()
    ctx.fastmcp.biggo_setting = setting

    search = await product_search(
        ctx=ctx,
        query="iphone",
    )

    assert isinstance(search, str)


def test_price_history_graph(setting: BigGoMCPSetting):

    ctx = MagicMock()
    ctx.fastmcp = MagicMock()
    ctx.fastmcp.biggo_setting = setting

    search = price_history_graph(
        ctx=ctx,
        history_id="tw_pmall_rakuten-nwdsl_6MONJRBOO",
    )

    assert isinstance(search, str)



from biggo_mcp_server.types.api_ret.product_search import ListItem, ProductSearchAPIRet
from biggo_mcp_server.types.responses import ProductSearchToolResponse, SpecSearchToolResponse


def test_product_search_resp():
    resp = ProductSearchToolResponse(product_search_result=ProductSearchAPIRet(
        list=[]))

    assert resp.reason is not None

    resp = ProductSearchToolResponse(product_search_result=ProductSearchAPIRet(
        list=[
            ListItem(
                oid="123",
                title="test",
                price=100,
                purl="https://www.test.com",
            )
        ]))

    assert resp.reason is None


def test_spec_search_resp():
    resp = SpecSearchToolResponse(hits=[])

    assert resp.reason is not None

    resp = SpecSearchToolResponse(hits=[{"some": "data"}])

    assert resp.reason is None
